@@ -9,14 +9,15 @@ use regex::Regex;
 use lazy_static::lazy_static;
 
 use crate::storage::Storage;
-use crate::shape::{ShapeTrait, Path, Ellipse};
+use crate::shape::{ShapeStored, path::Path, ellipse::Ellipse};
 use crate::color::Color;
 use crate::consts::DEFAULT_THICKNESS;
-use crate::point::Vec2DDrawing;
+use crate::point::{Point, Vec2DWorld};
+use crate::geom::Angle;
 
 mod path;
 
-use path::{parse_path, PathBuilder};
+use path::{parse_path, PathBuilder as SvgPathBuilder};
 
 lazy_static! {
     static ref CSS_COLOR_REGEX: Regex = Regex::new(r"(?x)#
@@ -36,6 +37,12 @@ lazy_static! {
         \s+
         (?P<y>-?\d+(\.\d+)?)
     ").unwrap();
+    static ref ANGLE_REGEX: Regex = Regex::new(r"(?x)
+        rotate\(
+            (?P<angle>-?\d+(\.\d*)?)
+            (\s+(-?\d+(\.\d*)?)\s+(-?\d+(\.\d*)?))?
+        \)
+    ").unwrap();
     static ref CSS_COLORS: HashMap<String, Color> = vec![
         ("aliceblue".into(), Color::from_int_rgb(0xF0, 0xF8, 0xFF)),
         ("antiquewhite".into(), Color::from_int_rgb(0xFA, 0xEB, 0xD7)),
@@ -193,6 +200,7 @@ pub enum Error {
     CouldntUnderstandColor(String),
     ParseFloatError(std::num::ParseFloatError),
     PointDoesntMatchRegex,
+    AngleDoesntMatchRegex(String),
     MissingAttribute(String),
     PathParseError(path::ParseError),
 }
@@ -252,12 +260,12 @@ impl FromStr for Color {
     }
 }
 
-impl FromStr for Vec2DDrawing {
+impl FromStr for Vec2DWorld {
     type Err = Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         if let Some(captures) = POINT_REGEX.captures(s) {
-            Ok(Vec2DDrawing::new(
+            Ok(Vec2DWorld::new(
                 captures["x"].parse::<f64>()?,
                 captures["y"].parse::<f64>()?,
             ))
@@ -267,6 +275,18 @@ impl FromStr for Vec2DDrawing {
     }
 }
 
+impl FromStr for Angle {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Angle, Self::Err> {
+        if let Some(angle) = ANGLE_REGEX.captures(s) {
+            Ok(Angle::from_degrees(angle["angle"].parse()?))
+        } else {
+            Err(Error::AngleDoesntMatchRegex(s.into()))
+        }
+    }
+}
+
 fn css_attrs_as_hashmap(attrs: &str) -> HashMap<&str, &str> {
     attrs.split(';').filter_map(|s| {
         let pieces: Vec<_> = s.split(':').collect();
@@ -279,6 +299,12 @@ fn css_attrs_as_hashmap(attrs: &str) -> HashMap<&str, &str> {
     }).collect()
 }
 
+fn xml_attrs_as_hashmap(attrs: Vec<OwnedAttribute>) -> HashMap<String, String> {
+    attrs.into_iter().map(|a| {
+        (a.name.local_name, a.value)
+    }).collect()
+}
+
 impl Path {
     pub fn from_xml_attributes(style: &str, path: &str) -> Result<Path, Error> {
         let attrs = css_attrs_as_hashmap(style);
@@ -305,65 +331,56 @@ impl Path {
                 1.0
             };
 
-        let mut builder = PathBuilder::new();
+        let mut builder = SvgPathBuilder::new();
 
         parse_path(path, &mut builder)?;
 
-        Ok(Path::with_params(color.with_alpha(alpha), builder.into_path(), thickness))
-    }
-}
-
-impl Ellipse {
-    pub fn from_xml_attributes(cx: &str, cy: &str, rx: &str, ry: &str, style: &str) -> Result<Ellipse, Error> {
-        let attrs = css_attrs_as_hashmap(style);
-
-        let color = if let Some(color_str) = attrs.get("stroke") {
-                color_str.parse()?
-            } else {
-                Color::white()
-            };
-
-        let thickness = if let Some(thickness_str) = attrs.get("stroke-width") {
-                thickness_str.parse()?
-            } else {
-                DEFAULT_THICKNESS
-            };
-
-        let alpha = if let Some(alpha_str) = attrs.get("stroke-opacity") {
-                alpha_str.parse()?
-            } else {
-                1.0
-            };
-
-        let center = Vec2DDrawing::new(cx.parse()?, cy.parse()?);
-        let radius = Vec2DDrawing::new(rx.parse()?, ry.parse()?).abs();
-
-        Ok(Ellipse::with_params([center-radius, center + radius], color.with_alpha(alpha), thickness))
+        Ok(Path::from_parts(builder.into_path(), color.with_alpha(alpha), thickness))
     }
 }
 
 trait Deserialize {
-    fn deserialize(attributes: Vec<OwnedAttribute>) -> Result<Box<dyn ShapeTrait>, Error>;
+    fn deserialize(attributes: HashMap<String, String>) -> Result<Box<dyn ShapeStored>, Error>;
 }
 
-impl Deserialize for Path {
-    fn deserialize(attributes: Vec<OwnedAttribute>) -> Result<Box<dyn ShapeTrait>, Error> {
-        let styleattr = attributes.iter().find(|a| a.name.local_name == "style").ok_or(Error::MissingAttribute("style".into()))?;
-        let dattr = attributes.iter().find(|a| a.name.local_name == "d").ok_or(Error::MissingAttribute("d".into()))?;
+impl Deserialize for Path{
+    fn deserialize(attributes: HashMap<String, String>) -> Result<Box<dyn ShapeStored>, Error> {
+        let styleattr = attributes.get("style").ok_or(Error::MissingAttribute("style".into()))?;
+        let dattr = attributes.get("d").ok_or(Error::MissingAttribute("d".into()))?;
 
-        Ok(Box::new(Path::from_xml_attributes(&styleattr.value, &dattr.value)?))
+        Ok(Box::new(Path::from_xml_attributes(&styleattr, &dattr)?))
     }
 }
 
 impl Deserialize for Ellipse {
-    fn deserialize(attributes: Vec<OwnedAttribute>) -> Result<Box<dyn ShapeTrait>, Error> {
-        let cx = attributes.iter().find(|a| a.name.local_name == "cx").ok_or(Error::MissingAttribute("style".into()))?;
-        let cy = attributes.iter().find(|a| a.name.local_name == "cy").ok_or(Error::MissingAttribute("style".into()))?;
-        let rx = attributes.iter().find(|a| a.name.local_name == "rx").ok_or(Error::MissingAttribute("style".into()))?;
-        let ry = attributes.iter().find(|a| a.name.local_name == "ry").ok_or(Error::MissingAttribute("style".into()))?;
-        let styleattr = attributes.iter().find(|a| a.name.local_name == "style").ok_or(Error::MissingAttribute("style".into()))?;
+    fn deserialize(attributes: HashMap<String, String>) -> Result<Box<dyn ShapeStored>, Error> {
+        let cx = attributes.get("cx").ok_or(Error::MissingAttribute("cx".into()))?;
+        let cy = attributes.get("cy").ok_or(Error::MissingAttribute("cy".into()))?;
+        let rx = attributes.get("rx").ok_or(Error::MissingAttribute("rx".into()))?;
+        let ry = attributes.get("ry").ok_or(Error::MissingAttribute("ry".into()))?;
+        let angle= attributes.get("transform").map(|s| s.as_str()).unwrap_or("rotate(0)");
+        let styleattr = attributes.get("style").ok_or(Error::MissingAttribute("style".into()))?;
 
-        Ok(Box::new(Ellipse::from_xml_attributes(&cx.value, &cy.value, &rx.value, &ry.value, &styleattr.value)?))
+        let attrs = css_attrs_as_hashmap(styleattr);
+
+        let color: Color = attrs.get("stroke").map(|c| c.parse()).transpose()?.unwrap_or_default();
+        let thickness = attrs.get("stroke-width").map(|s| s.parse()).transpose()?.unwrap_or(DEFAULT_THICKNESS);
+        let alpha = attrs.get("stroke-opacity").map(|s| s.parse()).transpose()?.unwrap_or(1.0);
+
+        let angle: Angle = angle.parse()?;
+
+        let center = Vec2DWorld::new(cx.parse()?, cy.parse()?);
+        let rx: f64 = rx.parse()?;
+        let ry: f64 = ry.parse()?;
+
+        Ok(Box::new(Ellipse::from_parts(
+            center,
+            rx,
+            ry,
+            angle,
+            color.with_alpha(alpha),
+            thickness,
+        )))
     }
 }
 
@@ -375,10 +392,10 @@ impl Storage {
         for e in parser {
             match e {
                 Ok(XmlEvent::StartElement { name, attributes, .. }) if name.local_name == "path" => {
-                    storage.add(Path::deserialize(attributes)?);
+                    storage.add(Path::deserialize(xml_attrs_as_hashmap(attributes))?);
                 }
                 Ok(XmlEvent::StartElement { name, attributes, .. }) if name.local_name == "ellipse" => {
-                    storage.add(Ellipse::deserialize(attributes)?);
+                    storage.add(Ellipse::deserialize(xml_attrs_as_hashmap(attributes))?);
                 }
                 Err(e) => {
                     return Err(e.into());
@@ -387,18 +404,16 @@ impl Storage {
             }
         }
 
-        storage.flush();
-
         Ok(storage)
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::storage::Storage;
-    use crate::shape::{Path, DrawCommand, ShapeTrait, path::PathCommand};
-    use crate::color::Color;
-    use crate::point::Vec2DDrawing;
+    use crate::shape::path::PathCommand;
+    use crate::draw_commands::DrawCommand;
+
+    use super::*;
 
     #[test]
     fn test_from_svg() {
@@ -451,21 +466,21 @@ mod tests {
         } = line.draw_commands() {
             assert_eq!(color, Color::from_rgb(0.5390625, 0.8828125, 0.203125));
             assert_eq!(commands, vec![
-                PathCommand::MoveTo(Vec2DDrawing::new(147.570312, 40.121094)),
-                PathCommand::LineTo(Vec2DDrawing::new(146.9375, 40.121094)),
-                PathCommand::LineTo(Vec2DDrawing::new(145.296875, 40.460938)),
-                PathCommand::LineTo(Vec2DDrawing::new(142.519531, 41.710938)),
-                PathCommand::LineTo(Vec2DDrawing::new(139.613281, 43.304688)),
-                PathCommand::LineTo(Vec2DDrawing::new(138.097656, 44.894531)),
-                PathCommand::LineTo(Vec2DDrawing::new(137.339844, 46.714844)),
-                PathCommand::LineTo(Vec2DDrawing::new(138.097656, 47.621094)),
-                PathCommand::LineTo(Vec2DDrawing::new(139.992188, 47.851562)),
-                PathCommand::LineTo(Vec2DDrawing::new(142.898438, 47.621094)),
-                PathCommand::LineTo(Vec2DDrawing::new(146.179688, 47.167969)),
-                PathCommand::LineTo(Vec2DDrawing::new(150.097656, 47.964844)),
-                PathCommand::LineTo(Vec2DDrawing::new(151.738281, 49.667969)),
-                PathCommand::LineTo(Vec2DDrawing::new(152.496094, 51.714844)),
-                PathCommand::LineTo(Vec2DDrawing::new(152.875, 53.191406)),
+                PathCommand::MoveTo(Vec2DWorld::new(147.570312, 40.121094)),
+                PathCommand::LineTo(Vec2DWorld::new(146.9375, 40.121094)),
+                PathCommand::LineTo(Vec2DWorld::new(145.296875, 40.460938)),
+                PathCommand::LineTo(Vec2DWorld::new(142.519531, 41.710938)),
+                PathCommand::LineTo(Vec2DWorld::new(139.613281, 43.304688)),
+                PathCommand::LineTo(Vec2DWorld::new(138.097656, 44.894531)),
+                PathCommand::LineTo(Vec2DWorld::new(137.339844, 46.714844)),
+                PathCommand::LineTo(Vec2DWorld::new(138.097656, 47.621094)),
+                PathCommand::LineTo(Vec2DWorld::new(139.992188, 47.851562)),
+                PathCommand::LineTo(Vec2DWorld::new(142.898438, 47.621094)),
+                PathCommand::LineTo(Vec2DWorld::new(146.179688, 47.167969)),
+                PathCommand::LineTo(Vec2DWorld::new(150.097656, 47.964844)),
+                PathCommand::LineTo(Vec2DWorld::new(151.738281, 49.667969)),
+                PathCommand::LineTo(Vec2DWorld::new(152.496094, 51.714844)),
+                PathCommand::LineTo(Vec2DWorld::new(152.875, 53.191406)),
             ]);
             assert_eq!(thickness, 3.0);
         } else {
@@ -495,13 +510,46 @@ mod tests {
     fn test_parse_point() {
         let p = " 152.496094 51.714844 ";
 
-        assert_eq!(p.parse::<Vec2DDrawing>().unwrap(), Vec2DDrawing::new(152.496094, 51.714844));
+        assert_eq!(p.parse::<Vec2DWorld>().unwrap(), Vec2DWorld::new(152.496094, 51.714844));
     }
 
     #[test]
     fn test_parse_alpha() {
         let line = Path::from_xml_attributes("fill:none;stroke-width:3;stroke-linecap:round;stroke-linejoin:round;stroke:#FF0000;stroke-opacity:0.8;stroke-miterlimit:10;", "M 10 10 L 20 20").unwrap();
 
-        assert_eq!(line.color(), "#FF0000".parse::<Color>().unwrap().with_alpha(0.8));
+        assert_eq!(line.color(), Color::red().with_alpha(0.8));
+    }
+
+    #[test]
+    fn can_deserialize_rotated_ellipse() {
+        let attrs: HashMap<String, String> = vec![
+            ("cx".into(), "20.4".into()),
+            ("cy".into(), "-30.5".into()),
+            ("rx".into(), "5.6".into()),
+            ("ry".into(), "3.5".into()),
+            ("transform".into(), "rotate(34.5)".into()),
+            ("style".into(), "stroke:#cabada;stroke-width:3.5;stroke-opacity:0.8".into()),
+        ].into_iter().collect();
+
+        let deserialized = Ellipse::deserialize(attrs).unwrap();
+
+        match deserialized.draw_commands() {
+            DrawCommand::Ellipse { center, semimajor, semiminor, angle, color, thickness } => {
+                assert_eq!(center, Vec2DWorld::new(20.4, -30.5));
+                assert_eq!(semimajor, 5.6);
+                assert_eq!(semiminor, 3.5);
+                assert_eq!(angle.degrees(), 34.5);
+                assert_eq!(color, Color::from_int_rgb(0xca, 0xba, 0xda).with_alpha(0.8));
+                assert_eq!(thickness, 3.5);
+            },
+            _ => panic!()
+        }
+    }
+
+    #[test]
+    fn can_deserialize_ellipse_serialization_test() {
+        let svg_data = include_str!("../res/serialize_ellipse.svg");
+
+        Storage::from_svg(svg_data).unwrap();
     }
 }
