@@ -1,5 +1,4 @@
 use std::fmt;
-use std::str::FromStr;
 
 use crate::point::{Vec2DWorld, Vec2DScreen};
 use crate::transform::Transform;
@@ -10,6 +9,7 @@ pub mod rectangle;
 pub mod polygon;
 pub mod circle;
 pub mod ellipse;
+pub mod three_point_circle;
 
 use super::color::Color;
 
@@ -18,6 +18,7 @@ use self::ellipse::ThreePointEllipseBuilder;
 use self::polygon::PolygonBuilder;
 use self::circle::CircleBuilder;
 use self::rectangle::AxisAlignedRectangleBuilder;
+use self::three_point_circle::CircleByThreePointsBuilder;
 
 /// Shapes handle input from the user, and they're responsible of notifying when
 /// they're finished, for example in the case of a polygon
@@ -29,36 +30,30 @@ pub enum ShapeFinished {
 
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub enum ShapeType {
+    Path,
+    Circle,
+    Ellipse,
+}
+
+#[derive(Debug, PartialEq, Copy, Clone)]
+pub enum ShapeTool {
     Path,
     Polygon,
     Rectangle,
-    Circle,
+    CircleByCenterAndPoint,
+    CircleByThreePoints,
     ThreePointEllipse,
 }
 
-impl ShapeType {
+impl ShapeTool {
     pub fn start(&self, color: Color, initial: Vec2DWorld, stroke: f64) -> Box<dyn ShapeBuilder> {
         match *self {
-            ShapeType::Path => Box::new(PathBuilder::start(color, initial, stroke)),
-            ShapeType::Rectangle => Box::new(AxisAlignedRectangleBuilder::start(color, initial, stroke)),
-            ShapeType::Polygon => Box::new(PolygonBuilder::start(color, initial, stroke)),
-            ShapeType::Circle => Box::new(CircleBuilder::start(color, initial, stroke)),
-            ShapeType::ThreePointEllipse => Box::new(ThreePointEllipseBuilder::start(color, initial, stroke)),
-        }
-    }
-}
-
-impl FromStr for ShapeType {
-    type Err = ();
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s {
-            "path" => Ok(ShapeType::Path),
-            "polygon" => Ok(ShapeType::Polygon),
-            "rectangle" => Ok(ShapeType::Rectangle),
-            "circle" => Ok(ShapeType::Circle),
-            "ellipse" => Ok(ShapeType::ThreePointEllipse),
-            _ => Err(()),
+            ShapeTool::Path => Box::new(PathBuilder::start(color, initial, stroke)),
+            ShapeTool::Rectangle => Box::new(AxisAlignedRectangleBuilder::start(color, initial, stroke)),
+            ShapeTool::Polygon => Box::new(PolygonBuilder::start(color, initial, stroke)),
+            ShapeTool::CircleByCenterAndPoint => Box::new(CircleBuilder::start(color, initial, stroke)),
+            ShapeTool::CircleByThreePoints => Box::new(CircleByThreePointsBuilder::start(color, initial, stroke)),
+            ShapeTool::ThreePointEllipse => Box::new(ThreePointEllipseBuilder::start(color, initial, stroke)),
         }
     }
 }
